<?php

/**
 * @license http://www.gnu.org/licenses/gpl-3.0.html  GNU GPL v3
 * @copyright  2019 Université Grenoble Alpes
 */

namespace mod_labnbook\fetch;

/**
 * Send queries to the LabnBook API as an authenticated LnB institution.
 */
class institution extends moodlefetcher
{
    const OK = 200;
    const ERROR_NETWORK = 0;
    const ERROR_TOKEN_NOTVALID = 401;
    const ERROR_LOGIN_UNKNOWN = 403;
    const ERROR_CREATION_FAILED = 500;

    /**
     * @return int Cf constants: OK, ERROR_*
     */
    public function loginUser(): int
    {
        try {
            $this->updateInfo();
            $this->fetch("POST", "/v1/auth/login");
        } catch (\Exception $e) {
            $error = $this->fetcher->getLastError();
            return (isset($error->code) ? (int) $error->code : 0);
        }
        return self::OK;
    }

    /**
     * Sends info to LabNbook
     */
    public function updateInfo()
    {
        $data = [
            'plugin_version' => get_config('mod_labnbook')->version,
            'platform_version' => 'Moodle ' . get_config('moodle')->release
        ];
        $this->fetch("POST", "/v1/extplatform", $data);
    }

    /**
     * Return a JWT token for the redirection page of LnB, so that the user ends on the user-binding page.
     *
     * @param string $returnUrl full URL, including protocol
     * @param string $role "learner" | "teacher"
     * @param StdClass $user
     * @return string JWT
     * @throws \moodle_exception
     */
    public function getBindingTokenForRedirect(string $returnUrl, string $role, $user): string
    {
        if ($role !== 'learner' && $role !== 'teacher') {
            throw new \moodle_exception("Invalid role value in fetch.institution.getBindingToken()");
        }
        $payload = [
            'path' => "/user/preBind",
            'forward' => [
                'role_ext' => $role,
                'return_to' => $returnUrl,
                'login' => $user->username,
                'user_name' => $user->lastname,
                'first_name' => $user->firstname,
                'inst_number' => $user->idnumber,
                'email' => $user->email,
            ],
        ];
        return $this->fetcher->encodeJwtToken('/v1/redirect', $payload);
    }

    protected function getSigningKey(): string
    {
        global $CFG;
        return $CFG->labnbook_api_key;
    }

    /**
     * deletes a teamconfig
     * @return boolean
     */
    public function deleteTeamconfig($teamconfigId)
    {
        return $this->fetch('POST', '/v1/teamconfig/'.$teamconfigId.'/delete');
    }
}
