<?php

/**
 * @license http://www.gnu.org/licenses/gpl-3.0.html  GNU GPL v3
 * @copyright  2019 Université Grenoble Alpes
 */

namespace mod_labnbook\fetch;

/**
 * Integrate Moodle needs to the composed fetcher.
 * It fills data and actions for the communication with LabnBook through a fetcher.
 * It will be extended by 2 classes, for exchanges as a LnB institution or a LnB user.
 */
abstract class moodlefetcher
{
    /**
     * @var fetcher
     */
    protected $fetcher;

    public function __construct()
    {
        global $CFG;
        $this->fetcher = new fetcher(
            $CFG->labnbook_api_url,
            $this->getSigningKey(),
            $this->getEnvironment()
        );
    }

    public function getRedirectUrl(): string
    {
        global $CFG;
        return \rtrim($CFG->labnbook_api_url, '/') . '/v1/redirect';
    }

    public function getLastError(): ?\Stdclass
    {
        return $this->fetcher->getLastError();
    }

    protected function getEnvironment(): environment
    {
        global $CFG, $USER;
        $env = new environment();
        $env->orig = "inst"; // default value
        $env->iss = (int) $CFG->labnbook_institution_id;
        $env->sub = (int) $USER->id;
        return $env;
    }

    protected function fetch($verb, $path, $payload = [])
    {
        global $SESSION;
        $response = $this->fetcher->fetch($verb, $path, $payload);
        if (!empty($response->auth)) {
            if (empty($SESSION->labnbook)) {
                $SESSION->labnbook = new \Stdclass();
            }
            $SESSION->labnbook->auth = $response->auth;
        }
        return isset($response->data) ? $response->data : $response;
    }

    /**
     * Query the LabNbook API, returns default value in case of error
     *
     * @param string $verb E.g. "GET"
     * @param string $urlPath E.g. "/v1/mission"
     * @param array $payload Default is empty. Else any assoc array.
     * @param mixed $default value to return in case of errors
     * @return mixed
     */
    protected function fetchIgnore(string $verb, string $urlPath, array $payload = [], $default = null)
    {
        try {
            return $this->fetch($verb, $urlPath, $payload);
        } catch (\Exception $e) {
            return $default;
        }

    }

    abstract protected function getSigningKey(): string;
}
