<?php

/**
 * @copyright   2019 Université Grenoble Alpes
 * @license     http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace mod_labnbook\local;

use mod_labnbook\fetch;

/* @var $DB \moodle_database */

/**
 * Misc functions.
 */
class helper
{
    const localUrlPrefix = 'lnb://';
    /**
     * Stop loading the current page if the current user has no LabnBook account.
     * Redirects on an account binding|creation if necessary.
     *
     * @global \Stdclass $COURSE
     * @throws \moodle_exception
     */
    public static function requireLabnbookAuthentication($destination = null)
    {
        global $CFG, $COURSE;
        if (fetch\user::isAuthenticated()) {
            return;
        }
        $institutionFetcher = new fetch\institution();
        switch ($institutionFetcher->loginUser()) {
            case fetch\institution::OK:
               return;
            case fetch\institution::ERROR_LOGIN_UNKNOWN:
                if (!$destination) {
                    $destination = $_SERVER['REQUEST_URI'];
                }
                $id = optional_param('id', 0, PARAM_INT);
                redirect(
                    $CFG->wwwroot."/mod/labnbook/user-binding.php?courseid={$COURSE->id}&id={$id}&return="
                    . \base64_encode($destination)
                );
                break;
            case fetch\institution::ERROR_TOKEN_NOTVALID:
               throw new \moodle_exception("Internal error with LabNbook");
            case fetch\institution::ERROR_NETWORK:
               throw new \moodle_exception("Unable to connected to LabNbook, network error");
            default:
               throw new \moodle_exception("Unknown error with LabNbook");
        }
    }

    /**
     * Grant teacher access to the LnB missions and classes linked to the course.
     *
     * @param int $courseId
     */
    public static function grantTeacherAccess($courseId)
    {
        global $DB, $SESSION;
        if (!empty($SESSION->labnbook->grantTeacherAccess[$courseId])) {
            return;
        }
        if (!fetch\user::isAuthenticated()) {
            return;
        }
        $ids = $DB->get_fieldset_select("labnbook", "labnbook_teamconfigid", "course = " . (int) $courseId);
        if ($ids) {
            $userFetcher = new fetch\user();
            $userFetcher->grantMissionsAccess($ids);
        }
        // use session to avoid repeating this query
        if (empty($SESSION->labnbook->grantTeacherAccess)) {
            $SESSION->labnbook->grantTeacherAccess = [];
        }
        $SESSION->labnbook->grantTeacherAccess[$courseId] = true;
    }

    public static function getActivityLocalUrlForRedirect($moduleinstance, $is_teacher)
    {
        $url = $is_teacher ? "/teacher/reports/?team_config=" : "/report/enter/";
        return self::localUrlPrefix . $url . $moduleinstance->labnbook_teamconfigid;
    }
}
