<?php

/**
 * The main mod_labnbook configuration form.
 *
 * @copyright   2019 Université Grenoble Alpes
 * @license     http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
use mod_labnbook\fetch;
use mod_labnbook\form\select_dynamic;
use mod_labnbook\local\helper;

\defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot . '/course/moodleform_mod.php');

// short-cut before loading the form, see `course/modedit.php`
helper::requireLabnbookAuthentication();
helper::grantTeacherAccess($COURSE->id);

/**
 * Module instance settings form.
 *
 * @copyright  2019 Université Grenoble Alpes
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class mod_labnbook_mod_form extends \moodleform_mod
{
    /**
     * Defines forms elements.
     */
    public function definition()
    {
        global $CFG;

        $mform = $this->_form;
        $creation_mode = $this->get_coursemodule() == null;
        $disabled = $creation_mode ? [] : array("readonly" => "readonly");

        $mform->addElement('header', 'external', "Paramètres LabNbook");

        $fetcher = new fetch\User();
        $createMissionUrl = $fetcher->getRedirectUrl() . "?token=" . $fetcher->getMissionTokenForRedirect();
        $mform->addElement(
            'static',
            'addMission',
            '',
            '<a href="#" onclick="window.open(\'' . $createMissionUrl . '\', \'_blank\')" >'
                . '<button type="button">'
                . get_string('addMission', 'mod_labnbook')
                . '</button></a>'
        );
        $mform->addHelpButton('addMission', 'addMission', 'mod_labnbook');
        if ($creation_mode) {
            $mform->addElement(
                'static',
                'refresh',
                '',
                '<button type="button" onclick="refreshMissions()" id="refresh-missions">'
                    . get_string('refresh', 'mod_labnbook')
                    . '</a>'
            );
            $mform->addHelpButton('refresh', 'refresh', 'mod_labnbook');
        }

        $missions = json_encode($fetcher->getMissions());
        $attrs = $disabled;
        $attrs['onchange'] = 'javascript: '
                    . 'document.querySelector("#id_name").value =  "["+getMission(this.value).code + "] " + getMission(this.value).name;'
                    // update the Atto text, but Atto is undocumented stuff (wiki pages are all dead)
                    . 'document.querySelector("#id_introeditoreditable").innerHTML = getMission(this.value).description;'
                    . 'document.querySelector("#id_introeditor").innerHTML = getMission(this.value).description;';
                    // TODO update the TinyMCE text. Is there a way to detect the editor plugin? A common API for pluggable editors?

        $dynamicSelect = new select_dynamic(
            'labnbook_missionid',
            get_string('mission', 'mod_labnbook'),
            ['0' => "-"],
            $attrs
        );
        $mform->addElement($dynamicSelect);
        $mform->setType('labnbook_missionid', PARAM_INT);
        $mform->addRule('labnbook_missionid', null, 'required', null, 'client');

        $fetchMissions = (new fetch\user())->getJsMissions();
        $mform->addElement(
            'html',
            <<<"EOJS"
<script>
var labnbookMissions = new Map();
function getMission(id) {
    return labnbookMissions.get(parseInt(id));
}

var refreshLock = 0;
function refreshMissions() {
    if (refreshLock) {
        return;
    }
    refreshLock = 1;

    var loading = document.createElement('span');
    loading.setAttribute('class', 'fa fa-spinner fa-pulse fa-fw');
    loading.setAttribute('id', 'labnbook-missions-loading');
    document.querySelector('#refresh-missions').parentNode.appendChild(loading);

    $fetchMissions.then(missions => {
        refreshLock = 0;
        document.querySelector('#labnbook-missions-loading').remove();
        updateMissionsHTML(missions);

    }).catch(() => {
        refreshLock = 0;
        document.querySelector('#labnbook-missions-loading').remove();
        // TODO: error, what shall we do?
        alert("Erreur...");
    });
}

function updateMissionsHTML(missions) {
        var mainContainer = document.querySelector('#fitem_id_labnbook_missionid');
        var container = document.querySelector('select[name="labnbook_missionid"]');
        container.innerHTML = '<option value="0">-</option>';
        var addMission = document.querySelector('#fitem_id_addMission');
        var refresh =  document.querySelector('#fitem_id_refresh');
        var otherFieldsets = document.querySelectorAll('fieldset:not(#id_external)');
        var errorDiv = document.querySelector('.fdescription');
        if (missions.length ==0 ) {
            addMission.style.display = 'flex';
            otherFieldsets.forEach((el) => {el.style.display = 'none';});
            mainContainer.style.display = 'none';
            errorDiv.style.display = 'none';
        } else {
            addMission.style.display = 'none';
            if (refresh) {
                refresh.style.display = 'none';
            }
            mainContainer.style.display = 'flex';
            otherFieldsets.forEach((el) => {el.style.display = 'block';});
            errorDiv.style.display = 'block';
        }
        for (m of missions) {
            labnbookMissions.set(m.id_mission, m);
            var e = document.createElement('option');
            e.setAttribute('value', m.id_mission);
            e.innerText = '[' + m.code + '] ' + m.name;
            container.appendChild(e);
        }
        if (typeof(window.missionid) != 'undefined') {
            document.querySelector('select[name="labnbook_missionid"]').value = window.missionid;
        }
}

function disabledReadonlySelectors() {
    document.querySelectorAll('select[readonly]').forEach(
      function (selector) {
        selector.style.cssText = "opacity:0.6"
selector
        for (let i =0; i< selector.options.length; i++) {
          let option = selector.options[i];
          if (!option.selected) {
            selector.removeChild(option);
            i = i -1;
          }
        };
      });
}

document.addEventListener("DOMContentLoaded", function(event) { 
    updateMissionsHTML($missions);
    var elem = document.querySelector('#id_submitbutton2');
    elem.parentNode.removeChild(elem);
    disabledReadonlySelectors();
})

</script>
EOJS
        );
        // Adding the "general" fieldset, where all the common settings are showed.

        $course = $this->get_course();
        $groups = groups_get_all_groups($course->id, 0, $course->defaultgroupingid);
        $group_options = ["error" => "-"];
        $group_options[0] = get_string('allparticipants');
        if (empty($groups)) {
            unset($group_options["error"]);
        }
        foreach ($groups as $g) {
            $group_options[$g->id] = $g->name;
        }
        $mform->addElement('select', 'groupid', get_string('group_for_activity', 'mod_labnbook'), $group_options, $disabled);
        $mform->addHelpButton('groupid', 'group_for_activity', 'mod_labnbook');
        $mform->setType('groupid', PARAM_INT);
        $mform->addRule("groupid", get_string("group_is_required", 'mod_labnbook'), 'numeric', null, 'client');
        $mform->addRule("groupid", get_string("group_is_required", 'mod_labnbook'), 'required', null, 'client');

        $mform->addElement('hidden', 'labnbook_teamconfigid', false);
        $mform->setType('labnbook_teamconfigid', PARAM_INT);
        $mform->addElement('header', 'general', get_string('general', 'form'));
        // Adding the standard "name" field.
        $mform->addElement('text', 'name', get_string('instancename', 'mod_labnbook'), ['size' => '64']);
        if (!empty($CFG->formatstringstriptags)) {
            $mform->setType('name', PARAM_TEXT);
        } else {
            $mform->setType('name', PARAM_CLEANHTML);
        }
        $mform->addRule('name', null, 'required', null, 'client');
        $mform->addRule('name', get_string('maximumchars', '', 255), 'maxlength', 255, 'client');
        $mform->addHelpButton('name', 'instancename', 'mod_labnbook');
        // TODO ajax that queries LNB for the number of participants in that group

        // Adding the standard "intro" and "introformat" fields.
        if ($CFG->branch >= 29) {
            $this->standard_intro_elements();
        } else {
            $this->add_intro_editor();
        }

        // Add standard grading elements.
        $this->standard_grading_coursemodule_elements();

        // Add standard elements.
        $this->standard_coursemodule_elements();

        // Add standard buttons.
        $this->add_action_buttons();
    }

    /**
     * Allows module to modify data returned by get_moduleinfo_data() or prepare_new_moduleinfo_data() before calling set_data()
     * This method is also called in the bulk activity completion form.
     *
     * Only available on moodleform_mod.
     *
     * @param array $default_values passed by reference
     */
    public function data_preprocessing(&$default_values){
        parent::data_preprocessing($default_values);
        // The labnbook missionid will not be correctly set as option list is retrieved by ajax
        // Thus we store the id and let the ajax use it later
        if (array_key_exists('labnbook_missionid', $default_values)) {
            echo '<script>window.missionid='.$default_values['labnbook_missionid'].';</script>';
            // Retrieve teamconfig data
            $fetcher = new fetch\User();
            $teamconfig = $fetcher->getTeamconfig($default_values['labnbook_teamconfigid']);
            foreach (['method', 'size_min', 'size_max', 'size_opt', 'teams_max'] as $field) {
                $default_values[$field]= $teamconfig->$field;
            }
        }
    }


    /**
     * Hook called once the form data has been loaded into the elements.
     */
    public function definition_after_data()
    {
        parent::definition_after_data();
        if ($this->_form->getElementValue('update')) {
            //$this->_form->freeze('labnbook_missionid');
        }
    }

    private function addInputInt($name, $required, $callback = null, $err_msg = null)
    {
        $this->_form->addElement('text', $name, get_string($name, 'mod_labnbook'), ['size' => '3', "value" => 0]);
        $this->_form->setType($name, PARAM_INT);
        if ($required) {
            if ($callback) {
                $type = 'callback';
                $opt = $callback;
            } else {
                $type = 'required';
                $opt = null;
            }
            $this->_form->addRule($name, $err_msg, $type, $opt, 'client');
            $this->_form->addHelpButton($name, $name, 'mod_labnbook');
        }
    }
}
