<?php

/**
 * Displays an instance of mod_labnbook according to the user's permission.
 *
 * @copyright   2019 Université Grenoble Alpes
 * @license     http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

/* @var $DB \moodle_database */

use mod_labnbook\fetch\user as fetchUser;
use mod_labnbook\local\helper;

require(__DIR__ . '/../../config.php');
require_once(__DIR__ . '/lib.php');

function renderAndDie($templateName, $templateData)
{
    global $PAGE, $OUTPUT, $moduleinstance, $course, $modulecontext, $cm;
    /* @var $PAGE moodle_page */
    $PAGE->set_url('/mod/labnbook/view.php', ['id' => $cm->id]);
    $PAGE->set_title(format_string($moduleinstance->name));
    $PAGE->set_heading(format_string($course->fullname));
    $PAGE->set_context($modulecontext);

    echo $OUTPUT->header();
    echo $OUTPUT->render_from_template("mod_labnbook/$templateName", $templateData);
    echo $OUTPUT->footer();
    die();
}

// Course_module ID, or
$id = optional_param('id', 0, PARAM_INT);

// ... module instance id.
$l  = optional_param('l', 0, PARAM_INT);

if ($id) {
    $cm             = get_coursemodule_from_id("labnbook", $id, 0, false, MUST_EXIST);
    $course         = $DB->get_record('course', ['id' => $cm->course], '*', MUST_EXIST);
    $moduleinstance = $DB->get_record(LABNBOOK_TABLE, ['id' => $cm->instance], '*', MUST_EXIST);
    unset($id);
} elseif ($l) {
    $moduleinstance = $DB->get_record(LABNBOOK_TABLE, ['id' => $l], '*', MUST_EXIST);
    $course         = $DB->get_record('course', ['id' => $moduleinstance->course], '*', MUST_EXIST);
    $cm             = get_coursemodule_from_instance("labnbook", $moduleinstance->id, $course->id, false, MUST_EXIST);
    unset($l);
} else {
    print_error(get_string('missingidandcmid', mod_labnbook));
}
/* @var $moduleinstance \mod_labnbook\ar\labnbook */

require_login($course, true, $cm);

$modulecontext = context_module::instance($cm->id);
$coursecontext = context_course::instance($course->id);

// the teacher has access to all the missions and classes used in this course
$isTeacher = has_capability('moodle/course:manageactivities', $coursecontext)
    || has_capability('mod/labnbook:addinstance', $coursecontext);


$templateData = [
    'title' => $moduleinstance->name,
    'description' =>  format_text($moduleinstance->intro, $moduleinstance->introformat, ['context' => $modulecontext]),
];
if (!$isTeacher) {
    // Retrieve user groups
    $groupingid = $cm->groupmode && $cm->groupingid ? $cm->groupingid : $course->defaultgroupingid;
    // Get all groups of the user for this course and the allowed groupingid if defined
    $groups = groups_get_all_groups($course->id, $USER->id, $groupingid);
    if ($moduleinstance->groupid > 0 && !array_key_exists($moduleinstance->groupid, $groups)) {
        $templateData['back'] = get_string('back');
        $templateData['error'] = get_string('not_part_of_group', 'labnbook');
        renderAndDie('view_error', $templateData);
    }
}

$destination = helper::getActivityLocalUrlForRedirect($moduleinstance, $isTeacher);

helper::requireLabnbookAuthentication($destination);

/*
$event = \mod_labnbook\event\course_module_viewed::create(array(
    'objectid' => $moduleinstance->id,
    'context' => $modulecontext
));
$event->add_record_snapshot('course', $course);
$event->add_record_snapshot(LABNBOOK_TABLE, $moduleinstance);
$event->trigger();
 */

$fetcher = new fetchUser();
$templateData['redirecturl'] = $fetcher->getRedirectUrl();
if ($isTeacher) {
    helper::grantTeacherAccess($course->id);
    $participants = get_participants_ids($course, $moduleinstance);
    $participants_linked_count = $fetcher->updateParticipants($course->id, $moduleinstance->groupid, $participants);
    $templateData['enrolled_users'] = mod_labnbook_get_string_plural('enrolled_users_lnb', 'labnbook', $participants_linked_count)
        . get_string('enrolled_users_on', 'labnbook')
        . mod_labnbook_get_string_plural('enrolled_users_moodle', 'labnbook', count($participants));
    // editing teacher view
    $templateName = 'view_teacher';
    $templateData['token_teaming'] = $fetcher->getTeamingTokenForRedirect($moduleinstance->labnbook_teamconfigid);
    $templateData['token_reports'] = $fetcher->getReportsTokenForRedirect($moduleinstance->labnbook_teamconfigid);
    $templateData['token_mission'] = $fetcher->getMissionEditTokenForRedirect($moduleinstance->labnbook_missionid);
    $templateData['token_classe'] = $fetcher->getClassEditTokenForRedirect($moduleinstance->labnbook_teamconfigid);
    $templateData['started_reports'] = mod_labnbook_get_string_plural('reports_started', 'labnbook', $fetcher->getNumStartedReports($moduleinstance->labnbook_teamconfigid));
    $submitted = $fetcher->getNumSubmittedReports($moduleinstance->labnbook_teamconfigid);
    $templateData['submitted_reports'] = $submitted ? mod_labnbook_get_string_plural('reports_submitted', 'labnbook', $submitted) : "";

    $templateData['open_team_config'] = (isset($_SESSION['labnbook_add']) && $_SESSION['labnbook_add'] == $moduleinstance->id);
    unset($_SESSION['labnbook_add']);
    $templateData['open_team_config_msg'] = addslashes(get_string('team_config_init', 'labnbook'));
    $templateData['main_links_text'] = get_string('lnb_management_main_links', 'labnbook');
    $templateData['other_links_text'] = get_string('lnb_management_links', 'labnbook');
} else {
    // student view
    $templateName = 'view_student';
    $templateData['token_report'] = $fetcher->getReportTokenForRedirect($moduleinstance->labnbook_teamconfigid);
}
renderAndDie($templateName, $templateData);
